%% Example_CheckLinearization
%
% This m-file checks the continuous-time linearizations constructed for 
% the nonlinear mass-spring-damper system. The dynamics for the nonlinear 
% MSD system are described further in NonlinearMSD.m.


%% System parameters
M = 100;        % # of masses
m = 1;          % Mass of individual blocks, kg
k1 = 0.5;       % Linear spring constant, N/m
k2 = 1;         % Cubic spring constant, N/m^3
b1 = 1;         % Linear damping constant, N/(m/s)
b2 = 0;         % Cubic damping constant, N/(m/s)^3

%% Equilibrium condition
% Simulate nonlinear system to find trim condition

% Parameter specifying equilibrium point, F=rho (in N)
rho = 0;
%rho = 10;

% Compute equilbrium point
xbar = ComputeMSDTrim(rho,M,m,k1,k2,b1,b2);

%% Compute Linearized System
Glin = LinearizedMSD(xbar,m,k1,k2,b1);

%% Compute Linearized and Nonlinear Step Responses

% Simulation time
Tf = 20;           % Final time, sec

% Input forces, N
ustep = 1e-2;
u = [0 ustep; Tf ustep];
F = [0 rho; Tf rho];

% Function handle to compute the system derivative
xdotfh = @(t,x) NonlinearMSD(t,x,m,k1,k2,b1,b2,F,u);

% Initial Conditions: Use trim condition computed above
x0 = xbar;

% Simulate Nonlinear System
[tnl,xnl]=ode45(xdotfh,[0 Tf],x0);
Nt = numel(tnl);
delxnl = xnl - repmat(xbar',[Nt,1]);
delynl = delxnl(:,M);

% Simulate Linearized System 
% STEP computes a unit step response. Scale output by ustep
[ylin,tlin,xlin]=step(Glin,Tf);
delxlin = xlin*ustep;
delylin = ylin*ustep;

%% Plot Results

% Deviations of output (block M position) from trim condition
figure(1); 
plot(tnl,delynl,'b',tlin,delylin,'r--')
xlabel('Time, sec');
ylabel('\delta_y, m');
legend('NL','Lin','Location','Best')
if exist('garyfyFigure','file')
    garyfyFigure;
end

% Deviations of several block positions from trim
blkidx = 98:100; % Select block positions to plot
Nplot = numel(blkidx);
figure(2);
for i=1:Nplot
    blki = blkidx(i);
    subplot(Nplot,1,i);
    plot(tnl,delxnl(:,blki),'b',tlin,delxlin(:,blki),'r--');
    ylabel(['\delta_x(' int2str(blki) ') , m']);
    grid on;
end
xlabel('Time, sec');
legend('NL','Lin','Location','Best')
if exist('garyfyFigure','file')
    garyfyFigure;
end
