%% Example_LPVDMD
%
% This m-file uses the LPVDMD method to construct a reduced order LPV 
% model for the nonlinear mass-spring-damper system. The dynamics for 
% the nonlinear MSD system are described further in NonlinearMSD.m.

%% System parameters
M = 100;        % # of masses
m = 1;          % Mass of individual blocks, kg
k1 = 0.5;       % Linear spring constant, N/m
k2 = 1;         % Cubic spring constant, N/m^3
b1 = 1;         % Linear damping constant, N/(m/s)
b2 = 0;         % Cubic damping constant, N/(m/s)^3


%% Data Collection

% Select Parameter Grid
rhogrid = 0:0.2:2;
Ngrid = numel(rhogrid);

% Time grid for snapshots
dt = 0.1;          % Time step, sec
Tf = 20;           % Final time, sec
tin = 0:dt:Tf;     % Time vector for snapshots, sec
Nt = numel(tin);

% Collect data at each grid point
xtrim = zeros(2*M,Ngrid);
X = zeros(2*M,Nt,Ngrid);
U = zeros(1,Nt,Ngrid);
Y = zeros(1,Nt,Ngrid);
for i=1:Ngrid
    % Grab current grid point
    rhoi = rhogrid(i);
    
    % Compute equlibrium condition
    xtrim(:,i) = ComputeMSDTrim(rhoi,M,m,k1,k2,b1,b2);
    
    % Excite nonlinear system with chirp input, u
    w0 = 0.1;
    wf = 1+rhoi;
    win = w0*(wf/w0).^(tin/Tf);
    uin = 1e-1*sin( win.*tin );
    u = [tin(:) uin(:)];
    F = [0 rhoi; Tf rhoi];
    xdotfh = @(t,x) NonlinearMSD(t,x,m,k1,k2,b1,b2,F,u);
    x0 = xtrim(:,i);
    [tnl,xnl]=ode45(xdotfh,tin,x0);
    
    % Collect snapshots
    X(:,:,i) = xnl'-repmat(x0,[1 Nt]);
    U(:,:,i) = uin;
    Y(:,:,i) = X(M,:,i);
end

%% Construct reduced-order model via LPV DMD method
Alg = 1;   % Alg = 1 (Single-Step) or  Alg=2 (Iterative)
if Alg==1;
    % Single-step Method
    Nqmax = 10;
    Etol = 0.05;
    [Gred,Q,Efitrel]=LPVDMD(X,U,Y,dt,Etol,Nqmax,Alg);
else
    % Iterative Method
    Nqmax = 10;
    Etol = 0.05;
    [Gred,Q,Efitrel]=LPVDMD(X,U,Y,dt,Etol,Nqmax,Alg);
end
Nq = size(Q,2);

%% Compare reduced order models with full-order linearizations
for i=[1 Ngrid]; %1:Ngrid
    % Linearization using modes selected using only data from rhogrid(i)
    % This is for comparison only and is not used to construct the
    % reduced-order LPV model.
    Nqi = 4;
    Gi=IODMD(X(:,:,i),U(:,:,i),Y(:,:,i),dt,Nqi);
    
    % Full-order linearization
    Glin = LinearizedMSD(xtrim(:,i),m,k1,k2,b1);
    
    % Compare full and reduced (IODMD) linearizations
    figure(1)
    bopt = bodeoptions;
    bopt.PhaseMatching='on';
    bode(Glin,'b',Gi,'r-.',Gred(:,:,i),'g:',bopt)
    legend('Full','Red(i)','Red(all)','Location','Best');
    title(['rho = ' num2str(rhogrid(i))]);
    if exist('garyfyFigure','file')
        garyfyFigure;
    end
    drawnow
end


%% Simulations: Compare Nonlinear and Linear Parameter-Varying Results

% Specify sinusoidal parameter trajectory rho(t)
Tf = 50;
tin = 0:dt:Tf;
Nt=numel(tin);

amp = -1;
bias = 1;
freq = 0.5;
rhoin = amp*cos(freq*tin)+bias;
F = [tin(:) rhoin(:)];

% Input Force, N
%u = [0 0; Tf 0];
%u = [tin(:) zeros(Nt,1)];
u = [tin(:) zeros(Nt,1)];    
u(tin>=25,2) = 0.1;

% Function handle to compute the system derivative
xdotfh = @(t,x) NonlinearMSD(t,x,m,k1,k2,b1,b2,F,u);

% Initial Conditions: Use trim condition associated with initial rho
x0=LPVLinearInterp(rhogrid,xtrim,rhoin(1));

% Simulate Nonlinear System
[tnl,xnl]=ode45(xdotfh,tin,x0);
ynl = xnl(:,M);

% Linear Simulation
delz = zeros(Nq,Nt);
dely = zeros(1,Nt);
zbar = zeros(Nq,Nt);
zbar(:,1) = Q'*x0;
ybar = zeros(1,Nt);
ybar(:,1) = x0(M);
for k=1:(Nt-1)
    % Trim Condition at time k+1
    xbar = LPVLinearInterp(rhogrid,xtrim,rhoin(k+1));
    zbar(:,k+1) = Q'*xbar;
    ybar(:,k+1) = xbar(M);
    delu = u(k,2);
    
    % Linearized state matrix at time k
    Fk = LPVLinearInterp(rhogrid,Gred.A,rhoin(k));
    Gk = LPVLinearInterp(rhogrid,Gred.B,rhoin(k));
    Hk = LPVLinearInterp(rhogrid,Gred.C,rhoin(k));
    Dk = LPVLinearInterp(rhogrid,Gred.D,rhoin(k));
    
    % Update State (Assumes delu =0)
    delz(:,k+1) = Fk*delz(:,k) + Gk*delu + ( zbar(:,k) - zbar(:,k+1) );
    
    % Update Output
    dely(:,k+1) = Hk*delz(:,k) + Dk*delu;
end
ylpv = dely + ybar;

% Plot:  Output (Block M position) vs time
figure(2)
plot(tnl,ynl,'b',tin,ylpv,'r-.',tin,ybar,'g:');
ylim([min(ybar) 1.1*max(ybar)]);
ylabel('Block M Position y, m');
xlabel('Time, sec');
legend('Nonlin','LPV','Trim','Location','Best');
grid on;
if exist('garyfyFigure','file')
    garyfyFigure;
end

% Plot: Several block positions from trim
blkidx = 98:100; % Select block positions to plot
Nplot = numel(blkidx);
figure(3);
for i=1:Nplot
    % Use modes and reduced order state to estimate position
    blki = blkidx(i);
    xlpv = Q(blki,:)*(delz+zbar);
    
    subplot(Nplot,1,i);
    plot(tnl,xnl(:,blki),'b',tin,xlpv,'r--');
    ylim([min(xlpv) 1.1*max(xlpv)]);
    ylabel(['x(' int2str(blki) ') , m']);
    grid on;
end
xlabel('Time, sec');
legend('Nonlin','Red','Location','Best')
if exist('garyfyFigure','file')
    garyfyFigure;
end
